@@ -785,9 +785,16 @@ func loadFuncBody(ctx *blockCtx, fn *gox.Func, body *ast.BlockStmt) {
 	cb.End()
 }
 
+func simplifyGopPackage(pkgPath string) string {
+	if strings.HasPrefix(pkgPath, "gop/") {
+		return "github.com/goplus/" + pkgPath
+	}
+	return pkgPath
+}
+
 func loadImport(ctx *blockCtx, spec *ast.ImportSpec) {
 	pkgPath := toString(spec.Path)
-	pkg := ctx.pkg.Import(pkgPath)
+	pkg := ctx.pkg.Import(simplifyGopPackage(pkgPath))
 	var name string
 	if spec.Name != nil {
 		name = spec.Name.Name
