@@ -94,7 +94,10 @@ func findGoModDir(dir string) (string, bool) {
 }
 
 func runCmd(cmd *base.Command, args []string) {
-	flag.Parse(args)
+	err := flag.Parse(args)
+	if err != nil {
+		log.Fatalln("parse input arguments failed:", err)
+	}
 	if flag.NArg() < 1 {
 		cmd.Usage(os.Stderr)
 	}
