@@ -1084,6 +1084,21 @@ func (p *printer) expr1(expr ast.Expr, prec1, depth int) {
 		p.print(token.RARROW, blank)
 		p.block(x.Body, 1)
 
+	case *ast.RangeExpr:
+		if x.Low != nil {
+			p.expr(x.Low)
+		}
+		p.print(token.COLON)
+		if x.High != nil {
+			p.expr(x.High)
+		}
+		if x.Colon2 != token.NoPos {
+			p.print(token.COLON)
+			if x.Last != nil {
+				p.expr(x.Last)
+			}
+		}
+
 	default:
 		log.Fatalf("unreachable %T\n", x)
 	}
